/*--------------------------------------------------------------------------------------
// Beispiel für die Messe. LED schalten mit Mobiltelefon                              //
// Autor: Erwin Büttner                                                               //
// Basiert: Basiert auf das Programm von http://www.martyncurrey.com/arduinobtcontrol/ //                                
// Datum: 21.01.2018                                                                 //
//----------------------------------------------------------------------------------  */


#include "FastLED.h"
#define NUM_LEDS 144
#define DATA_PIN 9

const byte numChars = 21;
char receivedChars[numChars];
boolean newData = false;
          
CRGB leds[NUM_LEDS];


//==================================================================================================================================
//==================================================================================================================================

void setup() {
   
  Serial.begin(9600);  
  FastLED.addLeds<WS2812B, DATA_PIN, GRB>(leds, NUM_LEDS).setCorrection(TypicalLEDStrip);
  delay(500);
  
}
 
//==================================================================================================================================
//==================================================================================================================================


void loop() 
{ 
    INTRO();
    LOGO();
    if (Serial.available() > 0)     
    {  
      recvWithStartEndMarkers(); 
    }
    if (newData) 
    { 
     newData = false;        
     if ( receivedChars[0] == 'S' )  
     {
        clearScreen();
        WhiteScreen();
        for(;;)
        {
          if (Serial.available() > 0)     
          {  
            recvWithStartEndMarkers(); 
          }
          if (newData) 
          { 
            newData = false;          
            if ( receivedChars[0] == 'N' )  
            {
              if ( receivedChars[1] == 'U' )   // UP
              { 
                  if ( receivedChars[2] == 'O'  && receivedChars[3] == 'N'  ) 
                  {
                    WhiteScreen();
                    UP();
                  }
              }
  
              if ( receivedChars[1] == 'D' )   // DOWN
              {
                  if ( receivedChars[2] == 'O'  && receivedChars[3] == 'N'  ) 
                  {
                    WhiteScreen();
                    DOWN();
                  }
  
              }
  
              if ( receivedChars[1] == 'L' )   // LEFT
              {
                  if ( receivedChars[2] == 'O'  && receivedChars[3] == 'N'  ) 
                  {
                    WhiteScreen();
                    LEFT();
                  }           
  
              }
  
              if ( receivedChars[1] == 'R' )   // RIGHT
              {
                  if ( receivedChars[2] == 'O'  && receivedChars[3] == 'N'  ) 
                  {
                    WhiteScreen();
                    RIGHT();
                  }          
              }
  
              if ( receivedChars[1] == 'E' )   // Exit
              {
                  break;
              }
  
              }
            }
            delay(5);    
          };
     }
        
    delay(5);
    
}
}

//==================================================================================================================================
//==================================================================================================================================

void recvWithStartEndMarkers() 
{
     static boolean recvInProgress = false;
     static byte ndx = 0;
     char startMarker = '[';
     char endMarker = ']';
 
     char rc;
 
     if (Serial.available() > 0) 
     {
          rc = Serial.read();
          if (recvInProgress == true) 
          {
               if (rc != endMarker) 
               {
                    receivedChars[ndx] = rc;
                    ndx++;
                    if (ndx >= numChars) { ndx = numChars - 1; }
               }
               else 
               {
                     receivedChars[ndx] = '\0'; // terminate the string
                     recvInProgress = false;
                     ndx = 0;
                     newData = true;
               }
          }
          else if (rc == startMarker) { recvInProgress = true; }
     }
}

//==================================================================================================================================
//==================================================================================================================================

void INTRO()
{
  for(unsigned char whiteLed = 0; whiteLed < NUM_LEDS; whiteLed++) 
  {
      if(whiteLed%8==0)
      {leds[whiteLed] = CRGB::White;};
      if(whiteLed%8==1)
      {leds[whiteLed] = CRGB::Blue;};      
      if(whiteLed%8==2)
      {leds[whiteLed] = CRGB::Red;};
      if(whiteLed%8==3)
      {leds[whiteLed] = CRGB::Green;};
      if(whiteLed%8==4)
      {leds[whiteLed] = CRGB::Purple;};
      if(whiteLed%8==5)
      {leds[whiteLed] = CRGB::Orange;};
      if(whiteLed%8==6)
      {leds[whiteLed] = CRGB::Brown;};
      if(whiteLed%8==7)
      {leds[whiteLed] = CRGB::Pink;};
            
      FastLED.show();
      delay(1);    
      leds[whiteLed] = CRGB::Black;
  };  
}

//==================================================================================================================================
//==================================================================================================================================

void LOGO()
{
  //1. Reihe
  //Blau+
  leds[36].setRGB(0,0,255);
  leds[37].setRGB(0,0,255);
  leds[38].setRGB(0,0,255);
  leds[39].setRGB(0,0,255);
  leds[40].setRGB(0,0,255);
  //Rot
  leds[43].setRGB(255,0,0);
  leds[44].setRGB(255,0,0);
  leds[45].setRGB(255,0,0);
  leds[46].setRGB(255,0,0);
  leds[47].setRGB(255,0,0);

  //2.Reihe
  //Rot
  leds[48].setRGB(255,0,0);
  leds[49].setRGB(255,0,0);
  leds[50].setRGB(255,0,0);
  leds[51].setRGB(255,0,0);
  leds[52].setRGB(255,0,0);
  //Blau
  leds[55].setRGB(0,0,255); 
  leds[56].setRGB(0,0,255);
  leds[57].setRGB(0,0,255);
  leds[58].setRGB(0,0,255);
  leds[59].setRGB(0,0,255);
  
  //3.Reihe
  //Blau
  leds[60].setRGB(0,0,255);
  leds[61].setRGB(0,0,255);
  leds[62].setRGB(0,0,255);
  leds[63].setRGB(0,0,255);
  leds[64].setRGB(0,0,255);
  //Rot
  leds[67].setRGB(255,0,0);
  leds[68].setRGB(255,0,0);
  leds[69].setRGB(255,0,0);
  leds[70].setRGB(255,0,0);
  leds[71].setRGB(255,0,0);
  
  //4.Reihe
  //Rot
  leds[72].setRGB(255,0,0);
  leds[73].setRGB(255,0,0);
  leds[74].setRGB(255,0,0);
  leds[75].setRGB(255,0,0);
  leds[76].setRGB(255,0,0);
  //Blau
  leds[79].setRGB(0,0,255); 
  leds[80].setRGB(0,0,255);
  leds[81].setRGB(0,0,255);
  leds[82].setRGB(0,0,255);
  leds[83].setRGB(0,0,255);
  
  //5.Reihe
  //Blau
  leds[84].setRGB(0,0,255);
  leds[85].setRGB(0,0,255);
  leds[86].setRGB(0,0,255);
  leds[87].setRGB(0,0,255);
  leds[88].setRGB(0,0,255);
  //Rot
  leds[91].setRGB(255,0,0);
  leds[92].setRGB(255,0,0);
  leds[93].setRGB(255,0,0);
  leds[94].setRGB(255,0,0);
  leds[95].setRGB(255,0,0);
  
  //6.Reihe
  //Rot
  leds[96].setRGB(255,0,0);
  leds[97].setRGB(255,0,0);
  leds[98].setRGB(255,0,0);
  leds[99].setRGB(255,0,0);
  leds[100].setRGB(255,0,0);
  //Blau
  leds[103].setRGB(0,0,255); 
  leds[104].setRGB(0,0,255);
  leds[105].setRGB(0,0,255);
  leds[106].setRGB(0,0,255);
  leds[107].setRGB(0,0,255);
  FastLED.show();
  delay(10);
}


//==================================================================================================================================
//==================================================================================================================================

void UP()
{
  for(unsigned char i=0; i<12;i++) 
  {
     leds[i] = CRGB::Green;
  };
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

void DOWN()
{
  for(unsigned char i=132; i<144;i++) 
  {
     leds[i] = CRGB::Green;
  };
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

void RIGHT()
{
  leds[11] = CRGB::Green;
  leds[12] = CRGB::Green;
  leds[35] = CRGB::Green;
  leds[36] = CRGB::Green;
  leds[59] = CRGB::Green;
  leds[60] = CRGB::Green;
  leds[83] = CRGB::Green;
  leds[84] = CRGB::Green;
  leds[107] = CRGB::Green;
  leds[108] = CRGB::Green;
  leds[131] = CRGB::Green;
  leds[132] = CRGB::Green;
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

void LEFT()
{
  leds[0] = CRGB::Green;
  leds[23] = CRGB::Green;
  leds[24] = CRGB::Green;
  leds[47] = CRGB::Green;
  leds[48] = CRGB::Green;
  leds[71] = CRGB::Green;
  leds[72] = CRGB::Green;
  leds[95] = CRGB::Green;
  leds[96] = CRGB::Green;
  leds[119] = CRGB::Green;
  leds[120] = CRGB::Green;
  leds[143] = CRGB::Green;
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

void clearScreen()
{
  for(unsigned char i=0; i<NUM_LEDS;i++) 
  {
     leds[i] = CRGB::Black;
  };
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

void WhiteScreen()
{
  for(unsigned char i=0; i<NUM_LEDS;i++) 
  {
     leds[i] = CRGB::White;
  };
  FastLED.show();
  delay(10);
}

//==================================================================================================================================
//==================================================================================================================================

  
 
 
 
 
 
